@@ -151,7 +151,10 @@ namespace ros
 
     double toSec()  const { return (double)sec + 1e-9*(double)nsec; };
     T& fromSec(double t) {
-      sec = (uint32_t)floor(t);
+      int64_t sec64 = (int64_t)floor(t);
+      if (sec64 < 0 || sec64 > UINT_MAX)
+        throw std::runtime_error("Duration is out of dual 32-bit range");
+      sec = (uint32_t)sec64;
       nsec = (uint32_t)boost::math::round((t-sec) * 1e9);
       // avoid rounding errors
       sec += (nsec / 1000000000ul);
@@ -256,8 +259,43 @@ namespace ros
     static bool isSystemTime() { return true; }
   };
 
+  /**
+   * \brief Time representation.  Always steady-clock time.
+   *
+   * Not affected by ROS time.
+   *
+   * ros::TimeBase provides most of its functionality.
+   */
+  class ROSTIME_DECL SteadyTime : public TimeBase<SteadyTime, WallDuration>
+  {
+    public:
+      SteadyTime()
+        : TimeBase<SteadyTime, WallDuration>()
+      {}
+
+      SteadyTime(uint32_t _sec, uint32_t _nsec)
+        : TimeBase<SteadyTime, WallDuration>(_sec, _nsec)
+      {}
+
+      explicit SteadyTime(double t) { fromSec(t); }
+
+      /**
+       * \brief Returns the current steady (monotonic) clock time.
+       */
+      static SteadyTime now();
+
+      /**
+       * \brief Sleep until a specific time has been reached.
+       * @return True if the desired sleep time was met, false otherwise.
+       */
+      static bool sleepUntil(const SteadyTime& end);
+
+      static bool isSystemTime() { return true; }
+  };
+
   ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const Time &rhs);
   ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const WallTime &rhs);
+  ROSTIME_DECL std::ostream &operator <<(std::ostream &os, const SteadyTime &rhs);
 }
 
 #endif // ROS_TIME_H
